@@ -201,15 +201,15 @@ It is used for TCP/IP devices."
 ;;;###tramp-autoload
 (defsubst tramp-adb-file-name-p (vec-or-filename)
   "Check if it's a VEC-OR-FILENAME for ADB."
-  (when-let* ((vec (tramp-ensure-dissected-file-name vec-or-filename)))
-    (string= (tramp-file-name-method vec) tramp-adb-method)))
+  (and-let* ((vec (tramp-ensure-dissected-file-name vec-or-filename))
+	     ((string= (tramp-file-name-method vec) tramp-adb-method)))))
 
 ;;;###tramp-autoload
 (defun tramp-adb-file-name-handler (operation &rest args)
   "Invoke the ADB handler for OPERATION.
 First arg specifies the OPERATION, second arg is a list of
 arguments to pass to the OPERATION."
-  (if-let ((fn (assoc operation tramp-adb-file-name-handler-alist)))
+  (if-let* ((fn (assoc operation tramp-adb-file-name-handler-alist)))
       (prog1 (save-match-data (apply (cdr fn) args))
 	(setq tramp-debug-message-fnh-function (cdr fn)))
     (prog1 (tramp-run-real-handler operation args)
@@ -616,7 +616,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		      (tramp-shell-quote-argument l2))
 		   "Error copying %s to %s" filename newname))
 
-	      (if-let ((tmpfile (file-local-copy filename)))
+	      (if-let* ((tmpfile (file-local-copy filename)))
 		  ;; Remote filename.
 		  (condition-case err
 		      (rename-file tmpfile newname ok-if-already-exists)
@@ -919,7 +919,7 @@ will be used."
 
 (defun tramp-adb-handle-get-remote-uid (vec id-format)
   "Like `tramp-get-remote-uid' for Tramp files.
- ID-FORMAT valid values are `string' and `integer'."
+ID-FORMAT valid values are `string' and `integer'."
   (tramp-adb-send-command vec "id")
   (tramp-read-id-output vec)
   (tramp-get-connection-property vec (format "uid-%s" id-format)))
@@ -1125,6 +1125,11 @@ connection if a previous connection has died for some reason."
 			 tramp-adb-program args)))
 		   (prompt (md5 (concat (prin1-to-string process-environment)
 					(current-time-string)))))
+
+	      ;; Set sentinel.  Initialize variables.
+	      (set-process-sentinel p #'tramp-process-sentinel)
+	      (tramp-post-process-creation p vec)
+
 	      ;; Wait for initial prompt.  On some devices, it needs
 	      ;; an initial RET, in order to get it.
               (sleep-for 0.1)
@@ -1133,10 +1138,6 @@ connection if a previous connection has died for some reason."
 	      (unless (process-live-p p)
 		(tramp-error vec 'file-error "Terminated!"))
 
-	      ;; Set sentinel.  Initialize variables.
-	      (set-process-sentinel p #'tramp-process-sentinel)
-	      (tramp-post-process-creation p vec)
-
 	      ;; Set connection-local variables.
 	      (tramp-set-connection-local-variables vec)
 
