@@ -1,6 +1,6 @@
 ;;; tramp-cache.el --- file information caching for Tramp  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2000, 2005-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2000, 2005-2024 Free Software Foundation, Inc.
 
 ;; Author: Daniel Pittman <daniel@inanna.danann.net>
 ;;         Michael Albinus <michael.albinus@gmx.de>
@@ -68,10 +68,10 @@
 
 ;; Some properties are handled special:
 ;;
-;; - "process-name", "process-buffer" and "first-password-request" are
-;;   not saved in the file `tramp-persistency-file-name', although
-;;   being connection properties related to a `tramp-file-name'
-;;   structure.
+;; - "process-name", "process-buffer", "first-password-request" and
+;;   "pw-spec" are not saved in the file
+;;   `tramp-persistency-file-name', although being connection
+;;   properties related to a `tramp-file-name' structure.
 ;;
 ;; - Reusable properties, which should not be saved, are kept in the
 ;;   process key retrieved by `tramp-get-process' (the main connection
@@ -144,7 +144,6 @@ If KEY is `tramp-cache-undefined', don't create anything, and return nil."
 (defun tramp-get-file-property (key file property &optional default)
   "Get the PROPERTY of FILE from the cache context of KEY.
 Return DEFAULT if not set."
-  ;; Unify localname.  Remove hop from `tramp-file-name' structure.
   (setq key (tramp-file-name-unify key file))
   (if (eq key tramp-cache-undefined) default
     (let* ((hash (tramp-get-hash-table key))
@@ -191,7 +190,6 @@ Return DEFAULT if not set."
 (defun tramp-set-file-property (key file property value)
   "Set the PROPERTY of FILE to VALUE, in the cache context of KEY.
 Return VALUE."
-  ;; Unify localname.  Remove hop from `tramp-file-name' structure.
   (setq key (tramp-file-name-unify key file))
   (if (eq key tramp-cache-undefined) value
     (let ((hash (tramp-get-hash-table key)))
@@ -224,7 +222,6 @@ Return VALUE."
 ;;;###tramp-autoload
 (defun tramp-flush-file-property (key file property)
   "Remove PROPERTY of FILE in the cache context of KEY."
-  ;; Unify localname.  Remove hop from `tramp-file-name' structure.
   (setq key (tramp-file-name-unify key file))
   (unless (eq key tramp-cache-undefined)
     (remhash property (tramp-get-hash-table key))
@@ -237,9 +234,8 @@ Return VALUE."
   "Remove some properties of FILE's upper directory."
   (when (file-name-absolute-p file)
     ;; `file-name-directory' can return nil, for example for "~".
-    (when-let ((file (file-name-directory file))
-	       (file (directory-file-name file)))
-      ;; Unify localname.  Remove hop from `tramp-file-name' structure.
+    (when-let* ((file (file-name-directory file))
+		(file (directory-file-name file)))
       (setq key (tramp-file-name-unify key file))
       (unless (eq key tramp-cache-undefined)
 	(dolist (property (hash-table-keys (tramp-get-hash-table key)))
@@ -254,7 +250,6 @@ Return VALUE."
 (defun tramp-flush-file-properties (key file)
   "Remove all properties of FILE in the cache context of KEY."
   (let ((truename (tramp-get-file-property key file "file-truename")))
-    ;; Unify localname.  Remove hop from `tramp-file-name' structure.
     (setq key (tramp-file-name-unify key file))
     (unless (eq key tramp-cache-undefined)
       (tramp-message key 8 "%s" (tramp-file-name-localname key))
@@ -338,17 +333,15 @@ FILE must be a local file name on a connection identified via KEY."
   "Save PROPERTY, run BODY, reset PROPERTY.
 Preserve timestamps."
   (declare (indent 3) (debug t))
-  `(progn
-     ;; Unify localname.  Remove hop from `tramp-file-name' structure.
-     (setf ,key (tramp-file-name-unify ,key ,file))
-     (let* ((hash (tramp-get-hash-table ,key))
-	    (cached (and (hash-table-p hash) (gethash ,property hash))))
-       (unwind-protect (progn ,@body)
-	 ;; Reset PROPERTY.  Recompute hash, it could have been flushed.
-         (setq hash (tramp-get-hash-table ,key))
-	 (if (consp cached)
-	     (puthash ,property cached hash)
-	   (remhash ,property hash))))))
+  `(let* ((key (tramp-file-name-unify ,key ,file))
+	  (hash (tramp-get-hash-table key))
+	  (cached (and (hash-table-p hash) (gethash ,property hash))))
+     (unwind-protect (progn ,@body)
+       ;; Reset PROPERTY.  Recompute hash, it could have been flushed.
+       (setq hash (tramp-get-hash-table key))
+       (if (consp cached)
+	   (puthash ,property cached hash)
+	 (remhash ,property hash)))))
 
 ;;;###tramp-autoload
 (defmacro with-tramp-saved-file-properties (key file properties &rest body)
@@ -356,22 +349,20 @@ Preserve timestamps."
 PROPERTIES is a list of file properties (strings).
 Preserve timestamps."
   (declare (indent 3) (debug t))
-  `(progn
-     ;; Unify localname.  Remove hop from `tramp-file-name' structure.
-     (setf ,key (tramp-file-name-unify ,key ,file))
-     (let* ((hash (tramp-get-hash-table ,key))
-	    (values
-	     (and (hash-table-p hash)
-		  (mapcar
-		   (lambda (property) (cons property (gethash property hash)))
-		   ,properties))))
-       (unwind-protect (progn ,@body)
-	 ;; Reset PROPERTIES.  Recompute hash, it could have been flushed.
-         (setq hash (tramp-get-hash-table ,key))
-	 (dolist (value values)
-	   (if (consp (cdr value))
-	       (puthash (car value) (cdr value) hash)
-	     (remhash (car value) hash)))))))
+  `(let* ((key (tramp-file-name-unify ,key ,file))
+	  (hash (tramp-get-hash-table key))
+	  (values
+	   (and (hash-table-p hash)
+		(mapcar
+		 (lambda (property) (cons property (gethash property hash)))
+		 ,properties))))
+     (unwind-protect (progn ,@body)
+       ;; Reset PROPERTIES.  Recompute hash, it could have been flushed.
+       (setq hash (tramp-get-hash-table key))
+       (dolist (value values)
+	 (if (consp (cdr value))
+	     (puthash (car value) (cdr value) hash)
+	   (remhash (car value) hash))))))
 
 ;;; -- Properties --
 
@@ -397,7 +388,8 @@ the connection, return DEFAULT."
 	       (not (and (processp key) (not (process-live-p key)))))
       (setq value cached
 	    cache-used t))
-    (tramp-message key 7 "%s %s; cache used: %s" property value cache-used)
+    (unless (eq key tramp-cache-version)
+      (tramp-message key 7 "%s %s; cache used: %s" property value cache-used))
     value))
 
 ;;;###tramp-autoload
@@ -410,11 +402,12 @@ is `tramp-cache-undefined', nothing is set.
 PROPERTY is set persistent when KEY is a `tramp-file-name' structure.
 Return VALUE."
   (setq key (tramp-file-name-unify key))
-  (when-let ((hash (tramp-get-hash-table key)))
+  (when-let* ((hash (tramp-get-hash-table key)))
     (puthash property value hash))
   (setq tramp-cache-data-changed
 	(or tramp-cache-data-changed (tramp-file-name-p key)))
-  (tramp-message key 7 "%s %s" property value)
+  (unless (eq key tramp-cache-version)
+    (tramp-message key 7 "%s %s" property value))
   value)
 
 ;;;###tramp-autoload
@@ -434,7 +427,7 @@ KEY identifies the connection, it is either a process or a
 used to cache connection properties of the local machine.
 PROPERTY is set persistent when KEY is a `tramp-file-name' structure."
   (setq key (tramp-file-name-unify key))
-  (when-let ((hash (tramp-get-hash-table key)))
+  (when-let* ((hash (tramp-get-hash-table key)))
     (remhash property hash))
   (setq tramp-cache-data-changed
 	(or tramp-cache-data-changed (tramp-file-name-p key)))
@@ -449,7 +442,7 @@ used to cache connection properties of the local machine."
   (setq key (tramp-file-name-unify key))
   (tramp-message
    key 7 "%s %s" key
-   (when-let ((hash (gethash key tramp-cache-data)))
+   (when-let* ((hash (gethash key tramp-cache-data)))
      (hash-table-keys hash)))
   (setq tramp-cache-data-changed
 	(or tramp-cache-data-changed (tramp-file-name-p key)))
@@ -473,38 +466,36 @@ used to cache connection properties of the local machine."
 (defmacro with-tramp-saved-connection-property (key property &rest body)
   "Save PROPERTY, run BODY, reset PROPERTY."
   (declare (indent 2) (debug t))
-  `(progn
-     (setf ,key (tramp-file-name-unify ,key))
-     (let* ((hash (tramp-get-hash-table ,key))
-	    (cached (and (hash-table-p hash)
-			 (gethash ,property hash tramp-cache-undefined))))
-       (unwind-protect (progn ,@body)
-	 ;; Reset PROPERTY.  Recompute hash, it could have been flushed.
-         (setq hash (tramp-get-hash-table ,key))
-	 (if (not (eq cached tramp-cache-undefined))
-	     (puthash ,property cached hash)
-	   (remhash ,property hash))))))
+  `(let* ((key (tramp-file-name-unify ,key))
+	  (hash (tramp-get-hash-table key))
+	  (cached (and (hash-table-p hash)
+		       (gethash ,property hash tramp-cache-undefined))))
+     (unwind-protect (progn ,@body)
+       ;; Reset PROPERTY.  Recompute hash, it could have been flushed.
+       (setq hash (tramp-get-hash-table key))
+       (if (not (eq cached tramp-cache-undefined))
+	   (puthash ,property cached hash)
+	 (remhash ,property hash)))))
 
 ;;;###tramp-autoload
 (defmacro with-tramp-saved-connection-properties (key properties &rest body)
   "Save PROPERTIES, run BODY, reset PROPERTIES.
 PROPERTIES is a list of file properties (strings)."
   (declare (indent 2) (debug t))
-  `(progn
-     (setf ,key (tramp-file-name-unify ,key))
-     (let* ((hash (tramp-get-hash-table ,key))
-	    (values
-	     (mapcar
-	      (lambda (property)
-		(cons property (gethash property hash tramp-cache-undefined)))
-	      ,properties)))
-       (unwind-protect (progn ,@body)
-	 ;; Reset PROPERTIES.  Recompute hash, it could have been flushed.
-	 (setq hash (tramp-get-hash-table ,key))
-	 (dolist (value values)
-	   (if (not (eq (cdr value) tramp-cache-undefined))
-	       (puthash (car value) (cdr value) hash)
-	     (remhash (car value) hash)))))))
+  `(let* ((key (tramp-file-name-unify ,key))
+	  (hash (tramp-get-hash-table key))
+	  (values
+	   (mapcar
+	    (lambda (property)
+	      (cons property (gethash property hash tramp-cache-undefined)))
+	    ,properties)))
+     (unwind-protect (progn ,@body)
+       ;; Reset PROPERTIES.  Recompute hash, it could have been flushed.
+       (setq hash (tramp-get-hash-table key))
+       (dolist (value values)
+	 (if (not (eq (cdr value) tramp-cache-undefined))
+	     (puthash (car value) (cdr value) hash)
+	   (remhash (car value) hash))))))
 
 ;;;###tramp-autoload
 (defun tramp-cache-print (table)
@@ -551,13 +542,13 @@ PROPERTIES is a list of file properties (strings)."
 (defun tramp-list-connections ()
   "Return all active `tramp-file-name' structs according to `tramp-cache-data'."
   (let ((tramp-verbose 0))
-    (delq nil (mapcar
-	       (lambda (key)
-		 (and (tramp-file-name-p key)
-		      (null (tramp-file-name-localname key))
-		      (tramp-connection-property-p key "process-buffer")
-		      key))
-	       (hash-table-keys tramp-cache-data)))))
+    (tramp-compat-seq-keep
+     (lambda (key)
+       (and (tramp-file-name-p key)
+	    (null (tramp-file-name-localname key))
+	    (tramp-connection-property-p key "process-buffer")
+	    key))
+     (hash-table-keys tramp-cache-data))))
 
 (defun tramp-dump-connection-properties ()
   "Write persistent connection properties into file \
@@ -590,7 +581,8 @@ PROPERTIES is a list of file properties (strings)."
 	       (progn
 		 (remhash "process-name" value)
 		 (remhash "process-buffer" value)
-		 (remhash "first-password-request" value))
+		 (remhash "first-password-request" value)
+		 (remhash "pw-spec" value))
 	     (remhash key cache)))
 	 cache)
 	;; Dump it.
