@@ -1,6 +1,6 @@
 ;;; tramp-cmds.el --- Interactive commands for Tramp  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2007-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2024 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -39,6 +39,8 @@
 (defvar mm-7bit-chars)
 (defvar reporter-eval-buffer)
 (defvar reporter-prompt-for-summary-p)
+(defvar tramp-repository-branch)
+(defvar tramp-repository-version)
 
 ;;;###tramp-autoload
 (defun tramp-change-syntax (&optional syntax)
@@ -70,12 +72,10 @@ SYNTAX can be one of the symbols `default' (default),
 ;;;###tramp-autoload
 (defun tramp-list-remote-buffers ()
   "Return a list of all buffers with remote `default-directory'."
-  (delq
-   nil
-   (mapcar
-    (lambda (x)
-      (when (tramp-tramp-file-p (tramp-get-default-directory x)) x))
-    (buffer-list))))
+  (tramp-compat-seq-keep
+   (lambda (x)
+     (when (tramp-tramp-file-p (tramp-get-default-directory x)) x))
+   (buffer-list)))
 
 ;;; Cleanup
 
@@ -281,7 +281,7 @@ non-nil."
   ;; Remove all buffers with a remote default-directory which fit the hook.
   (dolist (name (tramp-list-remote-buffers))
     (and (buffer-live-p (get-buffer name))
-	 (with-current-buffer (get-buffer name)
+	 (with-current-buffer name
 	   (run-hook-with-args-until-success 'tramp-cleanup-some-buffers-hook))
 	 (kill-buffer name))))
 
@@ -567,6 +567,7 @@ For details, see `tramp-rename-files'."
   :type '(choice (const "su")
 		 (const "sudo")
 		 (const "doas")
+		 (const "run0")
 		 (const "ksu")))
 
 (defun tramp-file-name-with-sudo (filename)
@@ -586,7 +587,7 @@ An alternative method could be chosen with `tramp-file-name-with-method'."
 	  (tramp-make-tramp-file-name
 	   (make-tramp-file-name
 	    :method tramp-file-name-with-method :localname localname)))
-	 ;; Remote file with multi-hop capable method..
+	 ;; Remote file with multi-hop capable method.
 	 ((tramp-multi-hop-p v)
 	  (tramp-make-tramp-file-name
 	   (make-tramp-file-name
@@ -631,7 +632,7 @@ If the buffer runs `dired', the buffer is reverted."
 ;;;###tramp-autoload
 (defun tramp-recompile-elpa-command-completion-p (_symbol _buffer)
   "A predicate for `tramp-recompile-elpa'.
-It is completed by \"M-x TAB\" only if package.el is loaded, and
+It is completed by `M-x TAB' only if package.el is loaded, and
 Tramp is an installed ELPA package."
   ;; We cannot apply `package-installed-p', this would also return the
   ;; builtin package.
@@ -645,7 +646,7 @@ This is needed if there are compatibility problems."
   ;; (declare (completion tramp-recompile-elpa-command-completion-p))
   (interactive)
   ;; We expect just one Tramp package is installed.
-  (when-let
+  (when-let*
       ((dir (tramp-compat-funcall
 	     'package-desc-dir
 	     (car (alist-get 'tramp (bound-and-true-p package-alist))))))
@@ -691,25 +692,25 @@ This is needed if there are compatibility problems."
      (format "tramp (%s %s/%s)" ; package name and version
 	     tramp-version tramp-repository-branch tramp-repository-version)
      (sort
-      (delq nil (mapcar
-	(lambda (x)
-	  (and x (boundp x) (not (get x 'tramp-suppress-trace))
-	       (cons x 'tramp-reporter-dump-variable)))
-	(append
-	 (mapcar #'intern (all-completions "tramp-" obarray #'boundp))
-	 ;; Non-Tramp variables of interest.
-	 '(shell-prompt-pattern
-	   backup-by-copying
-	   backup-by-copying-when-linked
-	   backup-by-copying-when-mismatch
-	   backup-by-copying-when-privileged-mismatch
-	   backup-directory-alist
-	   password-cache
-	   password-cache-expiry
-	   remote-file-name-inhibit-cache
-	   connection-local-profile-alist
-	   connection-local-criteria-alist
-	   file-name-handler-alist))))
+      (tramp-compat-seq-keep
+       (lambda (x)
+	 (and x (boundp x) (not (get x 'tramp-suppress-trace))
+	      (cons x 'tramp-reporter-dump-variable)))
+       (append
+	(mapcar #'intern (all-completions "tramp-" obarray #'boundp))
+	;; Non-Tramp variables of interest.
+	'(shell-prompt-pattern
+	  backup-by-copying
+	  backup-by-copying-when-linked
+	  backup-by-copying-when-mismatch
+	  backup-by-copying-when-privileged-mismatch
+	  backup-directory-alist
+	  password-cache
+	  password-cache-expiry
+	  remote-file-name-inhibit-cache
+	  connection-local-profile-alist
+	  connection-local-criteria-alist
+	  file-name-handler-alist)))
       (lambda (x y) (string< (symbol-name (car x)) (symbol-name (car y)))))
 
      'tramp-load-report-modules	; pre-hook
@@ -742,8 +743,8 @@ buffer in your bug report.
 
 (defun tramp-reporter-dump-variable (varsym mailbuf)
   "Pretty-print the value of the variable in symbol VARSYM."
-  (when-let ((reporter-eval-buffer reporter-eval-buffer)
-	     (val (buffer-local-value varsym reporter-eval-buffer)))
+  (when-let* ((reporter-eval-buffer reporter-eval-buffer)
+	      (val (buffer-local-value varsym reporter-eval-buffer)))
 
     (if (hash-table-p val)
 	;; Pretty print the cache.
@@ -792,12 +793,10 @@ buffer in your bug report.
 
   ;; Dump buffer local variables.
   (insert "\nlocal variables:\n================")
-  (dolist (buffer
-	   (delq nil
-		 (mapcar
-		  (lambda (b)
-                    (when (string-match-p "\\*tramp/" (buffer-name b)) b))
-		  (buffer-list))))
+  (dolist (buffer (tramp-compat-seq-keep
+		   (lambda (b)
+		     (when (string-match-p "\\*tramp/" (buffer-name b)) b))
+		   (buffer-list)))
     (let ((reporter-eval-buffer buffer)
 	  (elbuf (get-buffer-create " *tmp-reporter-buffer*")))
       (with-current-buffer elbuf
