@@ -1,6 +1,6 @@
 ;;; tramp-smb.el --- Tramp access functions for SMB servers  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2002-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2002-2024 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -340,15 +340,15 @@ This can be used to disable echo etc."
 ;;;###tramp-autoload
 (defsubst tramp-smb-file-name-p (vec-or-filename)
   "Check if it's a VEC-OR-FILENAME for SMB servers."
-  (when-let* ((vec (tramp-ensure-dissected-file-name vec-or-filename)))
-    (string= (tramp-file-name-method vec) tramp-smb-method)))
+  (and-let* ((vec (tramp-ensure-dissected-file-name vec-or-filename))
+	     ((string= (tramp-file-name-method vec) tramp-smb-method)))))
 
 ;;;###tramp-autoload
 (defun tramp-smb-file-name-handler (operation &rest args)
   "Invoke the SMB related OPERATION and ARGS.
 First arg specifies the OPERATION, second arg is a list of
 arguments to pass to the OPERATION."
-  (if-let ((fn (assoc operation tramp-smb-file-name-handler-alist)))
+  (if-let* ((fn (assoc operation tramp-smb-file-name-handler-alist)))
       (prog1 (save-match-data (apply (cdr fn) args))
 	(setq tramp-debug-message-fnh-function (cdr fn)))
     (prog1 (tramp-run-real-handler operation args)
@@ -428,9 +428,6 @@ arguments to pass to the OPERATION."
 	  (t2 (tramp-tramp-file-p newname))
 	  target)
       (with-parsed-tramp-file-name (if t1 dirname newname) nil
-	(unless (file-exists-p dirname)
-	  (tramp-error v 'file-missing dirname))
-
 	;; `copy-directory-create-symlink' exists since Emacs 28.1.
 	(if (and (bound-and-true-p copy-directory-create-symlink)
 		 (setq target (file-symlink-p dirname))
@@ -617,8 +614,8 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
       ;; `file-local-copy' returns a file name also for a local file
       ;; with `jka-compr-handler', so we cannot trust its result as
       ;; indication for a remote file name.
-      (if-let ((tmpfile
-		(and (tramp-tramp-file-p filename) (file-local-copy filename))))
+      (if-let* ((tmpfile
+		 (and (tramp-tramp-file-p filename) (file-local-copy filename))))
 	  ;; Remote filename.
 	  (condition-case err
 	      (rename-file tmpfile newname ok-if-already-exists)
@@ -769,7 +766,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	(forward-line)
 	(delete-region (point-min) (point)))
       (while (and (not (eobp)) (looking-at-p (rx bol (+ nonl) ":" (+ nonl))))
- 	(forward-line))
+	(forward-line))
       (delete-region (point) (point-max))
       (throw 'tramp-action 'ok))))
 
@@ -865,7 +862,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
   "Implement `file-attributes' for Tramp files using `stat' command."
   (tramp-message
    vec 5 "file attributes with stat: %s" (tramp-file-name-localname vec))
-  (let* (size id link uid gid atime mtime ctime mode inode)
+  (let (size id link uid gid atime mtime ctime mode inode)
     (when (tramp-smb-send-command
 	   vec (format "stat %s" (tramp-smb-shell-quote-localname vec)))
 
@@ -1068,17 +1065,15 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 
 	  ;; Filter entries.
 	  (setq entries
-		(delq
-		 nil
 		 (if (or wildcard (string-empty-p base))
 		     ;; Check for matching entries.
-		     (mapcar
+		     (tramp-compat-seq-keep
 		      (lambda (x)
 			(when (string-match-p (rx bol (literal base)) (nth 0 x))
 			  x))
 		      entries)
 		   ;; We just need the only and only entry FILENAME.
-		   (list (assoc base entries)))))
+		   (list (assoc base entries))))
 
 	  ;; Sort entries.
 	  (setq entries
@@ -1233,7 +1228,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	 ((car destination)
 	  (setq outbuf (current-buffer))))
 	;; stderr.
-	(tramp-message v 2 "%s" "STDERR not supported"))
+	(tramp-warning v "%s" "STDERR not supported"))
        ;; 't
        (destination
 	(setq outbuf (current-buffer))))
