@@ -1,6 +1,6 @@
 ;;; tramp-sshfs.el --- Tramp access functions via sshfs  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2021-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -67,9 +67,6 @@
                 (tramp-remote-shell-login   ("-l"))
                 (tramp-remote-shell-args    ("-c"))))
 
- (add-to-list 'tramp-connection-properties
-	      `(,(format "/%s:" tramp-sshfs-method) "direct-async-process" t))
-
  (tramp-set-completion-function
   tramp-sshfs-method tramp-completion-function-alist-ssh))
 
@@ -172,15 +169,15 @@ Operations not mentioned here will be handled by the default Emacs primitives.")
 ;;;###tramp-autoload
 (defsubst tramp-sshfs-file-name-p (vec-or-filename)
   "Check if it's a VEC-OR-FILENAME for sshfs."
-  (when-let* ((vec (tramp-ensure-dissected-file-name vec-or-filename)))
-    (string= (tramp-file-name-method vec) tramp-sshfs-method)))
+  (and-let* ((vec (tramp-ensure-dissected-file-name vec-or-filename))
+	     ((string= (tramp-file-name-method vec) tramp-sshfs-method)))))
 
 ;;;###tramp-autoload
 (defun tramp-sshfs-file-name-handler (operation &rest args)
   "Invoke the sshfs handler for OPERATION and ARGS.
 First arg specifies the OPERATION, second arg is a list of
 arguments to pass to the OPERATION."
-  (if-let ((fn (assoc operation tramp-sshfs-file-name-handler-alist)))
+  (if-let* ((fn (assoc operation tramp-sshfs-file-name-handler-alist)))
       (prog1 (save-match-data (apply (cdr fn) args))
 	(setq tramp-debug-message-fnh-function (cdr fn)))
     (prog1 (tramp-run-real-handler operation args)
@@ -253,96 +250,34 @@ arguments to pass to the OPERATION."
 (defun tramp-sshfs-handle-process-file
   (program &optional infile destination display &rest args)
   "Like `process-file' for Tramp files."
-  ;; The implementation is not complete yet.
-  (when (and (numberp destination) (zerop destination))
-    (error "Implementation does not handle immediate return"))
+  (tramp-skeleton-process-file program infile destination display args
+    (let ((coding-system-for-read 'utf-8-dos)) ; Is this correct?
 
-  (with-parsed-tramp-file-name (expand-file-name default-directory) nil
-    (let ((coding-system-for-read 'utf-8-dos) ; Is this correct?
-	  (command
+      (setq command
 	   (format
 	    "cd %s && exec %s"
 	    (tramp-unquote-shell-quote-argument localname)
 	    (mapconcat #'tramp-shell-quote-argument (cons program args) " ")))
-	  input tmpinput stderr tmpstderr outbuf)
-
-      ;; Determine input.
-      (if (null infile)
-	  (setq input (tramp-get-remote-null-device v))
-	(setq infile (file-name-unquote (expand-file-name infile)))
-	(if (tramp-equal-remote default-directory infile)
-	    ;; INFILE is on the same remote host.
-	    (setq input (tramp-unquote-file-local-name infile))
-	  ;; INFILE must be copied to remote host.
-	  (setq input (tramp-make-tramp-temp-file v)
-		tmpinput (tramp-make-tramp-file-name v input))
-	  (copy-file infile tmpinput t)))
       (when input (setq command (format "%s <%s" command input)))
-
-      ;; Determine output.
-      (cond
-       ;; Just a buffer.
-       ((bufferp destination)
-	(setq outbuf destination))
-       ;; A buffer name.
-       ((stringp destination)
-	(setq outbuf (get-buffer-create destination)))
-       ;; (REAL-DESTINATION ERROR-DESTINATION)
-       ((consp destination)
-	;; output.
-	(cond
-	 ((bufferp (car destination))
-	  (setq outbuf (car destination)))
-	 ((stringp (car destination))
-	  (setq outbuf (get-buffer-create (car destination))))
-	 ((car destination)
-	  (setq outbuf (current-buffer))))
-	;; stderr.
-	(cond
-	 ((stringp (cadr destination))
-	  (setcar (cdr destination) (expand-file-name (cadr destination)))
-	  (if (tramp-equal-remote default-directory (cadr destination))
-	      ;; stderr is on the same remote host.
-	      (setq stderr (tramp-unquote-file-local-name (cadr destination)))
-	    ;; stderr must be copied to remote host.  The temporary
-	    ;; file must be deleted after execution.
-	    (setq stderr (tramp-make-tramp-temp-file v)
-		  tmpstderr (tramp-make-tramp-file-name v stderr))))
-	 ;; stderr to be discarded.
-	 ((null (cadr destination))
-	  (setq stderr (tramp-get-remote-null-device v)))))
-       ;; 't
-       (destination
-	(setq outbuf (current-buffer))))
       (when stderr (setq command (format "%s 2>%s" command stderr)))
 
       (unwind-protect
-	  (apply
-	   #'tramp-call-process
-	   v (tramp-get-method-parameter v 'tramp-login-program)
-	   nil outbuf display
-	   (tramp-expand-args
-	    v 'tramp-login-args
-	    ?h (or (tramp-file-name-host v) "")
-	    ?u (or (tramp-file-name-user v) "")
-	    ?p (or (tramp-file-name-port v) "")
-            ?a "-t" ?l command))
+	  (setq ret
+		(apply
+		 #'tramp-call-process
+		 v (tramp-get-method-parameter v 'tramp-login-program)
+		 nil outbuf display
+		 (tramp-expand-args
+		  v 'tramp-login-args nil
+		  ?h (or (tramp-file-name-host v) "")
+		  ?u (or (tramp-file-name-user v) "")
+		  ?p (or (tramp-file-name-port v) "")
+		  ?a "-t" ?l command)))
 
 	;; Synchronize stderr.
 	(when tmpstderr
 	  (tramp-cleanup-connection v 'keep-debug 'keep-password)
-	  (tramp-fuse-unmount v))
-
-	;; Provide error file.
-	(when tmpstderr
-	  (rename-file tmpstderr (cadr destination) t))
-
-	;; Cleanup.  We remove all file cache values for the
-	;; connection, because the remote process could have changed
-	;; them.
-	(when tmpinput (delete-file tmpinput))
-	(when process-file-side-effects
-          (tramp-flush-directory-properties v "/"))))))
+	  (tramp-fuse-unmount v))))))
 
 (defun tramp-sshfs-handle-rename-file
     (filename newname &optional ok-if-already-exists)
@@ -424,7 +359,7 @@ connection if a previous connection has died for some reason."
 		(tramp-fuse-mount-spec vec)
 		(tramp-fuse-mount-point vec)
 		(tramp-expand-args
-		 vec 'tramp-mount-args
+		 vec 'tramp-mount-args nil
 		 ?p (or (tramp-file-name-port vec) ""))))))
       (tramp-error
        vec 'file-error "Error mounting %s" (tramp-fuse-mount-spec vec)))
@@ -445,6 +380,16 @@ connection if a previous connection has died for some reason."
     (with-tramp-connection-property
 	vec "gid-string" (tramp-get-local-gid 'string))))
 
+;; Default connection-local variables for Tramp.
+
+(connection-local-set-profile-variables
+ 'tramp-sshfs-connection-local-default-profile
+ '((tramp-direct-async-process t)))
+
+(connection-local-set-profiles
+ `(:application tramp :protocol ,tramp-sshfs-method)
+ 'tramp-sshfs-connection-local-default-profile)
+
 ;; `shell-mode' tries to open remote files like "/sshfs:user@host:~/.history".
 ;; This fails, because the tilde cannot be expanded.  Tell
 ;; `tramp-handle-expand-file-name' to tolerate this.
